<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <!-- The above 3 meta tags *must* come first in the head; any other head content must come *after* these tags -->
    <meta name="description" content="">
    <meta name="author" content="">
    <link rel="icon" href="{{ STATIC_URL }}favicon.ico">

    <title>HHypermap</title>

    <!-- Bootstrap core CSS -->
    <link href="{{ STATIC_URL }}bootstrap/css/bootstrap.min.css" rel="stylesheet">
    <!-- Bootstrap theme -->
    <link href="{{ STATIC_URL }}bootstrap/css/bootstrap-theme.min.css" rel="stylesheet">
    <!-- font awesome -->
    <link href="{{ STATIC_URL }}font-awesome/css/font-awesome.min.css" rel="stylesheet" type="text/css">

    <!-- css for HyperMap -->
    <link href="{{ STATIC_URL }}css/site.css" rel="stylesheet">
    <link href="{{ STATIC_URL }}css/morris.css" rel="stylesheet">

    <!-- HTML5 shim and Respond.js for IE8 support of HTML5 elements and media queries -->
    <!--[if lt IE 9]>
      <script src="https://oss.maxcdn.com/html5shiv/3.7.2/html5shiv.min.js"></script>
      <script src="https://oss.maxcdn.com/respond/1.4.2/respond.min.js"></script>
    <![endif]-->

    <link rel="search" type="application/opensearchdescription+xml" href="{% url 'opensearch_dispatch' %}" title="HHypermap Search"/>

    <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.12.0/jquery.min.js"></script>
    <script>window.jQuery || document.write('<script src="../../assets/js/vendor/jquery.min.js"><\/script>')</script>

    <link rel="stylesheet" href="https://ajax.googleapis.com/ajax/libs/jqueryui/1.11.4/themes/smoothness/jquery-ui.css">
    <script src="https://ajax.googleapis.com/ajax/libs/jqueryui/1.11.4/jquery-ui.min.js"></script>

    {% block head %}
    {% endblock %}

  </head>

  <body>

    <nav class="navbar navbar-inverse navbar-fixed-top">
      <div class="container">
        <div class="navbar-header">
          <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar" aria-expanded="false" aria-controls="navbar">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
          </button>
          <a class="navbar-brand" href="{% url 'index' %}">HHypermap</a>
        </div>
        <div id="navbar" class="collapse navbar-collapse">
          <ul class="nav navbar-nav">
            <li class="active"><a href="{% url 'index' %}">Home</a></li>
            {% if user.is_superuser %}
              <li><a href="{% url 'domains' %}">Domains</a></li>
              <li><a href="{% url 'celery_monitor' %}">Monitor</a></li>
              <li><a href="{% url 'admin:index' %}">Admin</a></li>
              <li><a href="{{ SEARCH_IP }}/#/hypermap" target="new">Solr</a></li>
            {% endif %}
          </ul>
        </div><!--/.nav-collapse -->
      </div>
    </nav>

    <div class="container">

      {% block content %}{% endblock %}

    </div><!-- /.container -->


    <!-- Bootstrap core JavaScript
    ================================================== -->
    <!-- Placed at the end of the document so the pages load faster -->
    <script src="{{ STATIC_URL }}bootstrap/js/bootstrap.min.js"></script>
    <!-- Raphael javascript core -->
    <script src="{{ STATIC_URL }}js/raphael-min.js"></script>
    <script src="{{ STATIC_URL }}js/morris.min.js"></script>
    <script src="{{ STATIC_URL }}js/publish.js"></script>
    {% block extrafoot %}{% endblock %}
  </body>
</html>
